BEGIN {
S["am__EXEEXT_FALSE"]=""
S["am__EXEEXT_TRUE"]="#"
S["LTLIBOBJS"]=""
S["subdirs"]=" extension"
S["GAWKLIBEXT"]="so"
S["LIBMPFR"]="-lmpfr -lgmp"
S["LIBREADLINE"]="-lreadline"
S["SOCKET_LIBS"]=""
S["ENABLE_EXTENSIONS_FALSE"]="#"
S["ENABLE_EXTENSIONS_TRUE"]=""
S["USE_PERSISTENT_MALLOC_FALSE"]="#"
S["USE_PERSISTENT_MALLOC_TRUE"]=""
S["LIBSIGSEGV_PREFIX"]=""
S["LTLIBSIGSEGV"]=""
S["LIBSIGSEGV"]=""
S["HAVE_LIBSIGSEGV"]="no"
S["LIBOBJS"]=""
S["TEST_CROSS_COMPILE_FALSE"]=""
S["TEST_CROSS_COMPILE_TRUE"]="#"
S["POSUB"]="po"
S["LTLIBINTL"]=""
S["LIBINTL"]=""
S["INTLLIBS"]=""
S["LTLIBICONV"]="-liconv"
S["LIBICONV"]="-liconv"
S["INTL_MACOSX_LIBS"]=""
S["XGETTEXT_EXTRA_OPTIONS"]=""
S["MSGMERGE_FOR_MSGFMT_OPTION"]="--for-msgfmt"
S["MSGMERGE"]="/usr/local/bin/msgmerge"
S["XGETTEXT_015"]="/usr/local/bin/xgettext"
S["XGETTEXT"]="/usr/local/bin/xgettext"
S["GMSGFMT_015"]="/usr/local/bin/msgfmt"
S["GMSGFMT"]="/usr/local/bin/msgfmt"
S["MSGFMT"]="/usr/local/bin/msgfmt"
S["GETTEXT_MACRO_VERSION"]="0.20"
S["USE_NLS"]="yes"
S["SED"]="/bin/sed"
S["ZOS_FAIL"]=""
S["acl_shlibext"]="so"
S["ac_ct_AR"]="ar"
S["AR"]="ar"
S["RANLIB"]="ranlib"
S["CPP"]="gcc -E"
S["LN_S"]="ln -s"
S["YFLAGS"]=""
S["YACC"]="bison -y"
S["EGREP"]="/bin/grep -E"
S["GREP"]="/bin/grep"
S["am__fastdepCC_FALSE"]="#"
S["am__fastdepCC_TRUE"]=""
S["CCDEPMODE"]="depmode=gcc3"
S["am__nodep"]="_no"
S["AMDEPBACKSLASH"]="\\"
S["AMDEP_FALSE"]="#"
S["AMDEP_TRUE"]=""
S["am__include"]="include"
S["DEPDIR"]=".deps"
S["OBJEXT"]="o"
S["EXEEXT"]=""
S["ac_ct_CC"]="gcc"
S["CPPFLAGS"]=""
S["LDFLAGS"]=" -no-pie"
S["CFLAGS"]="-g -O2 -DARRAYDEBUG -DYYDEBUG -DLOCALEDEBUG -DMEMDEBUG -Wall -fno-builtin -g3 -ggdb3"
S["CC"]="gcc"
S["host_os"]="linux-gnu"
S["host_vendor"]="pc"
S["host_cpu"]="x86_64"
S["host"]="x86_64-pc-linux-gnu"
S["build_os"]="linux-gnu"
S["build_vendor"]="pc"
S["build_cpu"]="x86_64"
S["build"]="x86_64-pc-linux-gnu"
S["pkgextensiondir"]="${pkglibdir}"
S["AM_BACKSLASH"]="\\"
S["AM_DEFAULT_VERBOSITY"]="1"
S["AM_DEFAULT_V"]="$(AM_DEFAULT_VERBOSITY)"
S["AM_V"]="$(V)"
S["CSCOPE"]="cscope"
S["ETAGS"]="etags"
S["CTAGS"]="ctags"
S["am__untar"]="$${TAR-tar} xf -"
S["am__tar"]="$${TAR-tar} chof - \"$$tardir\""
S["AMTAR"]="$${TAR-tar}"
S["am__leading_dot"]="."
S["SET_MAKE"]=""
S["AWK"]="gawk"
S["mkdir_p"]="$(MKDIR_P)"
S["MKDIR_P"]="/bin/mkdir -p"
S["INSTALL_STRIP_PROGRAM"]="$(install_sh) -c -s"
S["STRIP"]=""
S["install_sh"]="${SHELL} /home/arnold/Gnu/gawk/gawk.git/build-aux/install-sh"
S["MAKEINFO"]="${SHELL} '/home/arnold/Gnu/gawk/gawk.git/build-aux/missing' makeinfo"
S["AUTOHEADER"]="${SHELL} '/home/arnold/Gnu/gawk/gawk.git/build-aux/missing' autoheader"
S["AUTOMAKE"]="${SHELL} '/home/arnold/Gnu/gawk/gawk.git/build-aux/missing' automake-1.16"
S["AUTOCONF"]="${SHELL} '/home/arnold/Gnu/gawk/gawk.git/build-aux/missing' autoconf"
S["ACLOCAL"]="${SHELL} '/home/arnold/Gnu/gawk/gawk.git/build-aux/missing' aclocal-1.16"
S["VERSION"]="5.1.60"
S["PACKAGE"]="gawk"
S["CYGPATH_W"]="echo"
S["am__isrc"]=""
S["INSTALL_DATA"]="${INSTALL} -m 644"
S["INSTALL_SCRIPT"]="${INSTALL}"
S["INSTALL_PROGRAM"]="${INSTALL}"
S["target_alias"]=""
S["host_alias"]=""
S["build_alias"]=""
S["LIBS"]="-ldl -lm "
S["ECHO_T"]=""
S["ECHO_N"]="-n"
S["ECHO_C"]=""
S["DEFS"]="-DHAVE_CONFIG_H"
S["mandir"]="${datarootdir}/man"
S["localedir"]="${datarootdir}/locale"
S["libdir"]="${exec_prefix}/lib"
S["psdir"]="${docdir}"
S["pdfdir"]="${docdir}"
S["dvidir"]="${docdir}"
S["htmldir"]="${docdir}"
S["infodir"]="${datarootdir}/info"
S["docdir"]="${datarootdir}/doc/${PACKAGE_TARNAME}"
S["oldincludedir"]="/usr/include"
S["includedir"]="${prefix}/include"
S["runstatedir"]="${localstatedir}/run"
S["localstatedir"]="${prefix}/var"
S["sharedstatedir"]="${prefix}/com"
S["sysconfdir"]="${prefix}/etc"
S["datadir"]="${datarootdir}"
S["datarootdir"]="${prefix}/share"
S["libexecdir"]="${exec_prefix}/libexec"
S["sbindir"]="${exec_prefix}/sbin"
S["bindir"]="${exec_prefix}/bin"
S["program_transform_name"]="s,x,x,"
S["prefix"]="/usr/local"
S["exec_prefix"]="${prefix}"
S["PACKAGE_URL"]="https://www.gnu.org/software/gawk/"
S["PACKAGE_BUGREPORT"]="bug-gawk@gnu.org"
S["PACKAGE_STRING"]="GNU Awk 5.1.60"
S["PACKAGE_VERSION"]="5.1.60"
S["PACKAGE_TARNAME"]="gawk"
S["PACKAGE_NAME"]="GNU Awk"
S["PATH_SEPARATOR"]=":"
S["SHELL"]="/bin/bash"
S["am__quote"]=""
  for (key in S) S_is_set[key] = 1
  FS = ""

}
{
  line = $ 0
  nfields = split(line, field, "@")
  substed = 0
  len = length(field[1])
  for (i = 2; i < nfields; i++) {
    key = field[i]
    keylen = length(key)
    if (S_is_set[key]) {
      value = S[key]
      line = substr(line, 1, len) "" value "" substr(line, len + keylen + 3)
      len += length(value) + length(field[++i])
      substed = 1
    } else
      len += 1 + keylen
  }

  print line
}

